Package: csodata
Type: Package
Title: Download Data from the CSO 'PxStat' API
Version: 1.2.2
Date: 2021-12-10
Authors@R: c(
    person("Eoin", "Horgan", email = "eoin.horgan@cso.ie", role = "aut", comment = c(ORCID = "0000-0002-3446-6154")),
	person("Conor", "Crowley", email = "conor.crowley@cso.ie", role = c("aut", "cre")),
	person("Vytas", "Vaiciulis", email = "Vytas.Vaiciulis@cso.ie", role = c("aut")),
    person("Mervyn", "O'Luing", email = "mervyn.oluing@cso.ie", role = "aut"))
Maintainer: Conor Crowley <conor.crowley@cso.ie>
Description: Imports 'PxStat' data in JSON-stat format and (optionally) reshapes it into wide
	format. The Central Statistics Office (CSO) is the national statistical institute of Ireland
	and 'PxStat' is the CSOs online database of Official Statistics. This database contains current
	and historical data series compiled from CSO statistical releases and is accessed at
	<http://data.cso.ie>.
	The CSO 'PxStat' Application Programming Interface (API), which is accessed in this package, provides
	access to 'PxStat' data in JSON-stat format at <http://data.cso.ie>.
	This dissemination tool allows developers machine to machine access to CSO 'PxStat' data.
Imports: dplyr, httr, jsonlite, reshape2, rjstat, R.cache, sf,
        lubridate
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.1.1
Suggests: knitr, rmarkdown, tmap, viridisLite
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2021-12-10 14:06:24 UTC; crowleyco
Author: Eoin Horgan [aut] (<https://orcid.org/0000-0002-3446-6154>),
  Conor Crowley [aut, cre],
  Vytas Vaiciulis [aut],
  Mervyn O'Luing [aut]
Repository: CRAN
Date/Publication: 2021-12-10 15:00:05 UTC
